@@ -2992,6 +2992,10 @@ gui_mch_add_menu_item(
     if (parent->submenu_id == NULL)
 	return;
 
+    /* Don't add menu separator */
+    if (gui_is_separator(menu->name))
+	return;
+
     /* TODO: use menu->mnemonic and menu->actext */
     if (gui.vimWindow->Lock())
     {
