@@ -1809,6 +1809,10 @@ gui_mch_add_menu_item(menu, parent, idx)
 	if (parent->submenu_id == 0)
 		return;
 
+	/* Don't add menu separator */
+	if (gui_is_separator(menu->name))
+		return;
+
 	len = STRLEN(menu->dname);
 	name = alloc(len + 1);
 	if (name == NULL)
