@@ -132,6 +132,8 @@ static char	*zap_file[ZAP_STYLES] = {NULL, NULL};
 /* r_caddr format for current mode */
 static char	*zap_caddr[ZAP_STYLES] = {NULL, NULL};
 
+static void ro_remove_menu(int *menu);
+
 /*
  * Initialise all the ZapRedraw stuff.
  * Call this when changing font and after each mode change.
@@ -3026,7 +3028,7 @@ ro_build_menu(menu)
     return wimp_menu;
 }
 
-    void
+    static void
 ro_remove_menu(menu)
     int	    *menu;
 {
@@ -3256,7 +3258,7 @@ gui_mch_browse(saving, title, dflt, ext, initdir, filter)
 	fname = ((char *) block[7]);
 	strncpy(fname, dflt, 255);
 
-	if (xswi(OS_FSControl, 31, curbuf-> b_p_ft) & v_flag)
+	if (xswi(OS_FSControl, 31, curbuf->b_p_oft) & v_flag)
 	{
 	    filetype = 0xfff;
 	    strcpy(sprname + 5, "xxx");
