@@ -1138,7 +1138,7 @@ mch_chdir(dir)
  * (in curbuf) to the file's type.
  */
     void
-ro_read_filetype(file)
+mch_read_filetype(file)
     char_u  *file;
 {
     int	    type;
@@ -1164,7 +1164,7 @@ ro_read_filetype(file)
 }
 
     void
-ro_set_filetype(file, type)
+mch_set_filetype(file, type)
     char_u  *file;
     char_u  *type;
 {
