@@ -22,7 +22,7 @@
 #define EFN	0			/* Event flag */
 
 #ifdef PROTO
-# define DESC int
+# typedef int DESC
 #endif
 
 static void	vms_flushbuf(void);
@@ -445,7 +445,8 @@ mch_suspend(void)
     settmode(TMODE_COOK);		/* set to cooked mode */
 
     sprintf(symstr, "%08X", pid);
-    mch_setenv("VIMPID", symstr);
+    /* added 3rd parameter SK 980904 */
+    mch_setenv("VIMPID", symstr, 1);
 
     /* windgoto(wins[cw].rows-1,0);	*/
     /* putchar('\r');	*/
