@@ -71,6 +71,7 @@ FILE* fdDump = NULL;
 # define CONSOLE_CURSOR_INFO int
 # define LPCSTR char_u *
 # define WINBASEAPI
+# define INPUT_RECORD int
 #endif
 
 #ifndef USE_GUI_WIN32
@@ -840,6 +841,19 @@ mch_update_cursor(void)
     mch_set_cursor_shape(thickness);
 }
 
+/*
+ * Handle FOCUS_EVENT.
+ */
+    static void
+handle_focus_event(INPUT_RECORD ir)
+{
+    g_fJustGotFocus = ir.Event.FocusEvent.bSetFocus;
+    if (g_fJustGotFocus)
+	check_timestamps();
+    apply_autocmds(g_fJustGotFocus ? EVENT_FOCUSGAINED
+				: EVENT_FOCUSLOST, NULL, NULL, FALSE, curbuf);
+}
+
 /*
  * Wait until console input from keyboard or mouse is available,
  * or the time is up.
@@ -904,10 +918,9 @@ WaitForChar(long msec)
 #endif
 
 	    ReadConsoleInput(g_hConIn, &ir, 1, &cRecords);
-#ifdef USE_CLIPBOARD
+
 	    if (ir.EventType == FOCUS_EVENT)
-		g_fJustGotFocus = ir.Event.FocusEvent.bSetFocus;
-#endif
+		handle_focus_event(ir);
 	    else if (ir.EventType == WINDOW_BUFFER_SIZE_EVENT)
 		set_winsize(Rows, Columns, FALSE);
 #ifdef USE_MOUSE
@@ -969,13 +982,8 @@ tgetch(void)
 	    if (decode_key_event(&ir.Event.KeyEvent, &ch, &g_chPending, TRUE))
 		return ch;
 	}
-#ifdef USE_CLIPBOARD
 	else if (ir.EventType == FOCUS_EVENT)
-	{
-	    g_fJustGotFocus = ir.Event.FocusEvent.bSetFocus;
-	    /* TRACE("tgetch: Focus %d\n", g_fJustGotFocus); */
-	}
-#endif
+	    handle_focus_event(ir);
 	else if (ir.EventType == WINDOW_BUFFER_SIZE_EVENT)
 	{
 	    set_winsize(Rows, Columns, FALSE);
@@ -1631,6 +1639,10 @@ mch_isFullName(
 {
     char szName[_MAX_PATH+1];
 
+    /* A name like "d:/foo" is always absolute */
+    if (fname[0] && fname[1] == ':' && (fname[2] == '/' || fname[2] == '\\'))
+	return TRUE;
+
     mch_FullName(fname, szName, _MAX_PATH, FALSE);
 
     return strcoll(fname, szName) == 0;
@@ -2566,7 +2578,6 @@ mch_expandpath(
     return matches;
 }
 
-
 /*
  * The normal _chdir() does not change the default drive.  This one does.
  * Returning 0 implies success; -1 implies failure.
